#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/wait.h>
#include <signal.h>

static void sigchild_handler(int signo);
static int n_running_children;

int main(int argc, char** argv) {
  int i, n_children;
  pid_t pid;
  
  /* PROVARE AD ESEGUIRE QUESTO PROGRAMMA CON UN INTERO N_CHILDREN MOLTO GRANDE */
  if(argc != 2) {
    fprintf(stderr, "Uso: %s n_figli\n", argv[0]);
    exit(EXIT_FAILURE);
  }
  
  n_children = atoi(argv[1]);
  if(n_children <= 0) {
    fprintf(stderr, "Uso: %s n_figli, con n_figli > 0\n", argv[0]);
    exit(EXIT_FAILURE);
  }
  
  if (signal (SIGCHLD, SIG_IGN) != SIG_IGN) {
    if (signal(SIGCHLD, sigchild_handler) == SIG_ERR) 
      fprintf (stderr, "Errore nella configurazione dell'handler SIGCHLD.\n");
  }
  n_running_children = 0;
  
  for(i=0;i<n_children;i++) {
    n_running_children++;
    if( (pid=fork()) > 0) {
      printf("[padre] Generato figlio i=%d con PID %d\n", (i+1), pid);
    } else if (pid == 0) {
      execl("./figlio", "figlio", NULL);
    } else {
      n_running_children--;
      perror("Errore nella fork()");
    }
  } /* for */
  
  do {
    printf(".");fflush(stdout);
    sleep(1);

  } while (n_running_children > 0);
  
  exit(EXIT_SUCCESS);
}

void sigchild_handler(int signo) {
  int pid, status;
  printf("\n");
  pid = wait(&status);
  n_running_children--;
  if(pid < 0) {
    perror("Errore nella wait");
  } else if (WIFEXITED(status)) {
    printf("[padre] Processo figlio %d terminato normalmente - tramite exit - con status %d\n", pid, WEXITSTATUS(status));
  } else if(WIFSIGNALED(status)) {
    signo = WTERMSIG(status);
    printf("[padre] Processo figlio %d terminato a causa del segnale %d - %s.\n", pid, signo, sys_siglist[signo]);
  } else {
    printf("[padre] Processo figlio %d non terminato normalmente, ne' a causa di un segnale.\n", pid);
  } /* IF */
} 
